@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*  GMime
- *  Copyright (C) 2000-2014 Jeffrey Stedfast
+ *  Copyright (C) 2000-2017 Jeffrey Stedfast
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Lesser General Public License
@@ -165,15 +165,14 @@ static struct {
 };
 
 static GHashTable *iconv_charsets = NULL;
-static char **user_charsets = NULL;
 static char *locale_charset = NULL;
 static char *locale_lang = NULL;
+static int initialized = 0;
 
 #ifdef G_THREADS_ENABLED
-extern void _g_mime_charset_unlock (void);
-extern void _g_mime_charset_lock (void);
-#define CHARSET_UNLOCK() _g_mime_charset_unlock ()
-#define CHARSET_LOCK()   _g_mime_charset_lock ()
+static GMutex lock;
+#define CHARSET_UNLOCK() g_mutex_unlock (&lock);
+#define CHARSET_LOCK() g_mutex_lock (&lock);
 #else
 #define CHARSET_UNLOCK()
 #define CHARSET_LOCK()
@@ -188,9 +187,20 @@ extern void _g_mime_charset_lock (void);
 void
 g_mime_charset_map_shutdown (void)
 {
-	if (!iconv_charsets)
+	if (--initialized)
 		return;
 	
+#ifdef G_THREADS_ENABLED
+	if (glib_check_version (2, 37, 4) == NULL) {
+		/* The implementation of g_mutex_clear() prior
+		 * to glib 2.37.4 did not properly reset the
+		 * internal mutex pointer to NULL, so re-initializing
+		 * GMime would not properly re-initialize the mutexes.
+		 **/
+		g_mutex_clear (&lock);
+	}
+#endif
+	
 	g_hash_table_destroy (iconv_charsets);
 	iconv_charsets = NULL;
 	
@@ -256,9 +266,15 @@ g_mime_charset_map_init (void)
 	char *charset, *iconv_name, *locale;
 	int i;
 	
-	if (iconv_charsets)
+	initialized = MAX (initialized, 0);
+	
+	if (initialized++)
 		return;
 	
+#ifdef G_THREADS_ENABLED
+	g_mutex_init (&lock);
+#endif
+	
 	iconv_charsets = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, g_free);
 	
 	for (i = 0; known_iconv_charsets[i].charset != NULL; i++) {
@@ -345,11 +361,6 @@ g_mime_charset_map_init (void)
 const char *
 g_mime_locale_charset (void)
 {
-	CHARSET_LOCK ();
-	if (!iconv_charsets)
-		g_mime_charset_map_init ();
-	CHARSET_UNLOCK ();
-	
 	return locale_charset ? locale_charset : "iso-8859-1";
 }
 
@@ -359,16 +370,11 @@ g_mime_locale_charset (void)
  *
  * Gets the user's locale language code (or %NULL by default).
  *
- * Returns: the user's locale language code (or %NULL by default).
+ * Returns: (nullable): the user's locale language code (or %NULL by default).
  **/
 const char *
 g_mime_locale_language (void)
 {
-	CHARSET_LOCK ();
-	if (!iconv_charsets)
-		g_mime_charset_map_init ();
-	CHARSET_UNLOCK ();
-	
 	return locale_lang;
 }
 
@@ -381,8 +387,8 @@ g_mime_locale_language (void)
  * @charset. Currently only handles CJK and Russian/Ukranian
  * charset->lang mapping. Everything else will return %NULL.
  *
- * Returns: a language code that is specific to @charset, or %NULL on
- * fail.
+ * Returns: (nullable): a language code that is specific to @charset,
+ * or %NULL on fail.
  **/
 const char *
 g_mime_charset_language (const char *charset)
@@ -436,8 +442,6 @@ g_mime_charset_iconv_name (const char *charset)
 	strdown (name);
 	
 	CHARSET_LOCK ();
-	if (!iconv_charsets)
-		g_mime_charset_map_init ();
 	
 	iconv_name = g_hash_table_lookup (iconv_charsets, name);
 	if (iconv_name) {
@@ -783,8 +787,8 @@ charset_best_mask (unsigned int mask)
  *
  * Gets the best charset name based on the charset mask @charset.
  *
- * Returns: a pointer to a string containing the best charset name that
- * can represent the charset mask @charset.
+ * Returns: (nullable): the charset name that best represents the charset
+ * mask @charset or %NULL for us-ascii.
  **/
 const char *
 g_mime_charset_best_name (GMimeCharset *charset)
@@ -805,8 +809,8 @@ g_mime_charset_best_name (GMimeCharset *charset)
  *
  * Computes the best charset to use to encode this text buffer.
  *
- * Returns: the charset name best suited for the input text or %NULL if
- * it is US-ASCII safe.
+ * Returns: (nullable): the charset name best suited for the input text
+ * or %NULL if it is ascii-safe.
  **/
 const char *
 g_mime_charset_best (const char *inbuf, size_t inlen)
@@ -908,42 +912,3 @@ g_mime_charset_can_encode (GMimeCharset *mask, const char *charset, const char *
 	
 	return rc != (size_t) -1;
 }
-
-
-/**
- * g_mime_set_user_charsets:
- * @charsets: an array of user-preferred charsets
- *
- * Set a list of charsets for GMime to use as a hint for encoding and
- * decoding headers. The charset list should be in order of preference
- * (e.g. most preferred first, least preferred last).
- **/
-void
-g_mime_set_user_charsets (const char **charsets)
-{
-	if (user_charsets)
-		g_strfreev (user_charsets);
-	
-	if (charsets == NULL || charsets[0] == NULL) {
-		user_charsets = NULL;
-		return;
-	}
-	
-	user_charsets = g_strdupv ((char **) charsets);
-}
-
-
-/**
- * g_mime_user_charsets:
- *
- * Get the list of user-preferred charsets set with
- * g_mime_set_user_charsets().
- *
- * Returns: (array zero-terminated=1) (transfer none): an array of
- * user-set charsets or %NULL if none set.
- **/
-const char **
-g_mime_user_charsets (void)
-{
-	return (const char **) user_charsets;
-}
