@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*  GMime
- *  Copyright (C) 2000-2014 Jeffrey Stedfast
+ *  Copyright (C) 2000-2017 Jeffrey Stedfast
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Lesser General Public License
@@ -26,6 +26,7 @@
 #include <string.h>
 
 #include "gmime-crypto-context.h"
+#include "gmime-common.h"
 #include "gmime-error.h"
 
 
@@ -46,45 +47,37 @@ static void g_mime_crypto_context_finalize (GObject *object);
 
 static GMimeDigestAlgo crypto_digest_id (GMimeCryptoContext *ctx, const char *name);
 
-static const char *crypto_digest_name (GMimeCryptoContext *ctx, GMimeDigestAlgo );
+static const char *crypto_digest_name (GMimeCryptoContext *ctx, GMimeDigestAlgo digest);
 
 static const char *crypto_get_signature_protocol (GMimeCryptoContext *ctx);
-
 static const char *crypto_get_encryption_protocol (GMimeCryptoContext *ctx);
-
 static const char *crypto_get_key_exchange_protocol (GMimeCryptoContext *ctx);
 
-static int crypto_set_retrieve_session_key (GMimeCryptoContext *ctx, gboolean retrieve_session_key,
-					    GError **err);
-static gboolean crypto_get_retrieve_session_key (GMimeCryptoContext *ctx);
-
-static int crypto_sign (GMimeCryptoContext *ctx, const char *userid,
-			GMimeDigestAlgo digest, GMimeStream *istream,
-			GMimeStream *ostream, GError **err);
+static int crypto_sign (GMimeCryptoContext *ctx, gboolean detach, const char *userid, 
+			GMimeStream *istream, GMimeStream *ostream, GError **err);
 	
-static GMimeSignatureList *crypto_verify (GMimeCryptoContext *ctx, GMimeDigestAlgo digest,
+static GMimeSignatureList *crypto_verify (GMimeCryptoContext *ctx, GMimeVerifyFlags flags,
 					  GMimeStream *istream, GMimeStream *sigstream,
-					  GError **err);
+					  GMimeStream *ostream, GError **err);
 	
-static int crypto_encrypt (GMimeCryptoContext *ctx, gboolean sign,
-			   const char *userid, GMimeDigestAlgo digest,
-			   GPtrArray *recipients, GMimeStream *istream,
-			   GMimeStream *ostream, GError **err);
+static int crypto_encrypt (GMimeCryptoContext *ctx, gboolean sign, const char *userid,
+			   GMimeEncryptFlags flags, GPtrArray *recipients,
+			   GMimeStream *istream, GMimeStream *ostream,
+			   GError **err);
 
-static GMimeDecryptResult *crypto_decrypt (GMimeCryptoContext *ctx, GMimeStream *istream,
+static GMimeDecryptResult *crypto_decrypt (GMimeCryptoContext *ctx, GMimeDecryptFlags flags,
+					   const char *session_key, GMimeStream *istream,
 					   GMimeStream *ostream, GError **err);
 
-static GMimeDecryptResult *crypto_decrypt_session (GMimeCryptoContext *ctx, const char *session_key,
-						   GMimeStream *istream, GMimeStream *ostream,
-						   GError **err);
-
 static int crypto_import_keys (GMimeCryptoContext *ctx, GMimeStream *istream,
 			       GError **err);
 
-static int crypto_export_keys (GMimeCryptoContext *ctx, GPtrArray *keys,
+static int crypto_export_keys (GMimeCryptoContext *ctx, const char *keys[],
 			       GMimeStream *ostream, GError **err);
 
 
+static GHashTable *type_hash = NULL;
+
 static GObjectClass *parent_class = NULL;
 
 
@@ -107,6 +100,8 @@ g_mime_crypto_context_get_type (void)
 		};
 		
 		type = g_type_register_static (G_TYPE_OBJECT, "GMimeCryptoContext", &info, 0);
+		
+		type_hash = g_hash_table_new_full (g_mime_strcase_hash, g_mime_strcase_equal, g_free, NULL);
 	}
 	
 	return type;
@@ -128,14 +123,11 @@ g_mime_crypto_context_class_init (GMimeCryptoContextClass *klass)
 	klass->verify = crypto_verify;
 	klass->encrypt = crypto_encrypt;
 	klass->decrypt = crypto_decrypt;
-	klass->decrypt_session = crypto_decrypt_session;
 	klass->import_keys = crypto_import_keys;
 	klass->export_keys = crypto_export_keys;
 	klass->get_signature_protocol = crypto_get_signature_protocol;
 	klass->get_encryption_protocol = crypto_get_encryption_protocol;
 	klass->get_key_exchange_protocol = crypto_get_key_exchange_protocol;
-	klass->get_retrieve_session_key = crypto_get_retrieve_session_key;
-	klass->set_retrieve_session_key = crypto_set_retrieve_session_key;
 }
 
 static void
@@ -151,8 +143,55 @@ g_mime_crypto_context_finalize (GObject *object)
 }
 
 
+void
+g_mime_crypto_context_shutdown (void)
+{
+	g_hash_table_destroy (type_hash);
+	type_hash = NULL;
+}
+
+
 /**
- * g_mime_crypto_context_set_request_password:
+ * g_mime_crypto_context_register: (skip)
+ * @protocol: crypto protocol
+ * @callback: a #GMimeCryptoContextNewFunc
+ *
+ * Registers the callback for the specified @protocol.
+ **/
+void
+g_mime_crypto_context_register (const char *protocol, GMimeCryptoContextNewFunc callback)
+{
+	g_return_if_fail (protocol != NULL);
+	g_return_if_fail (callback != NULL);
+	
+	g_hash_table_replace (type_hash, g_strdup (protocol), callback);
+}
+
+
+/**
+ * g_mime_crypto_context_new:
+ * @protocol: the crypto protocol
+ *
+ * Creates a new crypto context for the specified @protocol.
+ *
+ * Returns: (nullable): a newly allocated #GMimeCryptoContext.
+ **/
+GMimeCryptoContext *
+g_mime_crypto_context_new (const char *protocol)
+{
+	GMimeCryptoContextNewFunc func;
+	
+	g_return_val_if_fail (protocol != NULL, NULL);
+	
+	if (!(func = g_hash_table_lookup (type_hash, protocol)))
+		return NULL;
+	
+	return func ();
+}
+
+
+/**
+ * g_mime_crypto_context_set_request_password: (skip)
  * @ctx: a #GMimeCryptoContext
  * @request_passwd: a callback function for requesting a password
  *
@@ -168,71 +207,6 @@ g_mime_crypto_context_set_request_password (GMimeCryptoContext *ctx, GMimePasswo
 }
 
 
-static gboolean
-crypto_get_retrieve_session_key (GMimeCryptoContext *ctx)
-{
-	return FALSE;
-}
-
-/**
- * g_mime_crypto_context_get_retrieve_session_key:
- * @ctx: a #GMimeCryptoContext
- *
- * Gets whether or not the @ctx is configured to retrieve a session
- * key during decryption (see g_mime_decrypt_result_get_session_key()).
- *
- * Returns: %TRUE if the @ctx is configured to retrieve a session key
- * or %FALSE otherwise.
- **/
-gboolean
-g_mime_crypto_context_get_retrieve_session_key (GMimeCryptoContext *ctx)
-{
-	g_return_val_if_fail (GMIME_IS_CRYPTO_CONTEXT (ctx), FALSE);
-	
-	return GMIME_CRYPTO_CONTEXT_GET_CLASS (ctx)->get_retrieve_session_key (ctx);
-}
-
-static int
-crypto_set_retrieve_session_key (GMimeCryptoContext *ctx, gboolean retrieve_session_key,
-				 GError **err)
-{
-	if (!retrieve_session_key)
-		return 0;
-	
-	g_set_error (err, GMIME_ERROR, GMIME_ERROR_NOT_SUPPORTED,
-		     "Session key retrieval is not supported by this crypto context");
-	
-	return -1;
-}
-
-/**
- * g_mime_crypto_context_set_retrieve_session_key:
- * @ctx: a #GMimeCryptoContext
- * @retrieve_session_key: whether to retrieve session keys during decryption
- * @err: a #GError
- *
- * Configures whether @ctx should produce a session key during future
- * decryption operations (see
- * g_mime_decrypt_result_get_session_key()).
- *
- * Returns: %0 on success or %-1 on fail.
- **/
-int
-g_mime_crypto_context_set_retrieve_session_key (GMimeCryptoContext *ctx,
-						gboolean retrieve_session_key,
-						GError **err)
-{
-	if (!GMIME_IS_CRYPTO_CONTEXT (ctx)) {
-		g_set_error (err, GMIME_ERROR, GMIME_ERROR_NOT_SUPPORTED,
-			     "Not a GMimeCryptoContext, can't set retrieve_session_key");
-		return -1;
-	}
-	
-	return GMIME_CRYPTO_CONTEXT_GET_CLASS (ctx)->set_retrieve_session_key (ctx, retrieve_session_key, err);
-}
-
-
-
 static GMimeDigestAlgo
 crypto_digest_id (GMimeCryptoContext *ctx, const char *name)
 {
@@ -247,7 +221,7 @@ crypto_digest_id (GMimeCryptoContext *ctx, const char *name)
  * Gets the function used by the @ctx for requesting a password from
  * the user.
  *
- * Returns: a #GMimePasswordRequestFunc or %NULL if not set.
+ * Returns: (nullable): a #GMimePasswordRequestFunc or %NULL if not set.
  **/
 GMimePasswordRequestFunc
 g_mime_crypto_context_get_request_password (GMimeCryptoContext *ctx)
@@ -291,7 +265,7 @@ crypto_digest_name (GMimeCryptoContext *ctx, GMimeDigestAlgo digest)
  *
  * Gets the digest name based on the digest id @digest.
  *
- * Returns: the equivalent digest name or %NULL on fail.
+ * Returns: (nullable): the equivalent digest name or %NULL on fail.
  **/
 const char *
 g_mime_crypto_context_digest_name (GMimeCryptoContext *ctx, GMimeDigestAlgo digest)
@@ -315,7 +289,7 @@ crypto_get_signature_protocol (GMimeCryptoContext *ctx)
  *
  * Gets the signature protocol for the crypto context.
  *
- * Returns: the signature protocol or %NULL if not supported.
+ * Returns: (nullable): the signature protocol or %NULL if not supported.
  **/
 const char *
 g_mime_crypto_context_get_signature_protocol (GMimeCryptoContext *ctx)
@@ -339,7 +313,7 @@ crypto_get_encryption_protocol (GMimeCryptoContext *ctx)
  *
  * Gets the encryption protocol for the crypto context.
  *
- * Returns: the encryption protocol or %NULL if not supported.
+ * Returns: (nullable): the encryption protocol or %NULL if not supported.
  **/
 const char *
 g_mime_crypto_context_get_encryption_protocol (GMimeCryptoContext *ctx)
@@ -363,7 +337,7 @@ crypto_get_key_exchange_protocol (GMimeCryptoContext *ctx)
  *
  * Gets the key exchange protocol for the crypto context.
  *
- * Returns: the key exchange protocol or %NULL if not supported.
+ * Returns: (nullable): the key exchange protocol or %NULL if not supported.
  **/
 const char *
 g_mime_crypto_context_get_key_exchange_protocol (GMimeCryptoContext *ctx)
@@ -375,11 +349,11 @@ g_mime_crypto_context_get_key_exchange_protocol (GMimeCryptoContext *ctx)
 
 
 static int
-crypto_sign (GMimeCryptoContext *ctx, const char *userid, GMimeDigestAlgo digest,
+crypto_sign (GMimeCryptoContext *ctx, gboolean detach, const char *userid,
 	     GMimeStream *istream, GMimeStream *ostream, GError **err)
 {
-	g_set_error (err, GMIME_ERROR, GMIME_ERROR_NOT_SUPPORTED,
-		     "Signing is not supported by this crypto context");
+	g_set_error_literal (err, GMIME_ERROR, GMIME_ERROR_NOT_SUPPORTED,
+			     "Signing is not supported by this crypto context");
 	
 	return -1;
 }
@@ -388,35 +362,34 @@ crypto_sign (GMimeCryptoContext *ctx, const char *userid, GMimeDigestAlgo digest
 /**
  * g_mime_crypto_context_sign:
  * @ctx: a #GMimeCryptoContext
+ * @detach: %TRUE if @ostream should be the detached signature; otherwise, %FALSE
  * @userid: private key to use to sign the stream
- * @digest: digest algorithm to use
  * @istream: input stream
  * @ostream: output stream
  * @err: a #GError
  *
  * Signs the input stream and writes the resulting signature to the output stream.
  *
- * Returns: the #GMimeDigestAlgo used on success (useful if @digest is
- * specified as #GMIME_DIGEST_ALGO_DEFAULT) or %-1 on fail.
+ * Returns: the #GMimeDigestAlgo used on success or %-1 on fail.
  **/
 int
-g_mime_crypto_context_sign (GMimeCryptoContext *ctx, const char *userid, GMimeDigestAlgo digest,
+g_mime_crypto_context_sign (GMimeCryptoContext *ctx, gboolean detach, const char *userid,
 			    GMimeStream *istream, GMimeStream *ostream, GError **err)
 {
 	g_return_val_if_fail (GMIME_IS_CRYPTO_CONTEXT (ctx), -1);
 	g_return_val_if_fail (GMIME_IS_STREAM (istream), -1);
 	g_return_val_if_fail (GMIME_IS_STREAM (ostream), -1);
 	
-	return GMIME_CRYPTO_CONTEXT_GET_CLASS (ctx)->sign (ctx, userid, digest, istream, ostream, err);
+	return GMIME_CRYPTO_CONTEXT_GET_CLASS (ctx)->sign (ctx, detach, userid, istream, ostream, err);
 }
 
 
 static GMimeSignatureList *
-crypto_verify (GMimeCryptoContext *ctx, GMimeDigestAlgo digest, GMimeStream *istream,
-	       GMimeStream *sigstream, GError **err)
+crypto_verify (GMimeCryptoContext *ctx, GMimeVerifyFlags flags, GMimeStream *istream, GMimeStream *sigstream,
+	       GMimeStream *ostream, GError **err)
 {
-	g_set_error (err, GMIME_ERROR, GMIME_ERROR_NOT_SUPPORTED,
-		     "Verifying is not supported by this crypto context");
+	g_set_error_literal (err, GMIME_ERROR, GMIME_ERROR_NOT_SUPPORTED,
+			     "Verifying is not supported by this crypto context");
 	
 	return NULL;
 }
@@ -425,36 +398,38 @@ crypto_verify (GMimeCryptoContext *ctx, GMimeDigestAlgo digest, GMimeStream *ist
 /**
  * g_mime_crypto_context_verify:
  * @ctx: a #GMimeCryptoContext
- * @digest: digest algorithm used, if known
+ * @flags: a #GMimeVerifyFlags
  * @istream: input stream
- * @sigstream: optional detached-signature stream
+ * @sigstream: (nullable): detached-signature stream
+ * @ostream: (nullable): output stream for use with encapsulated signature input streams
  * @err: a #GError
  *
- * Verifies the signature. If @istream is a clearsigned stream,
- * you should pass %NULL as the sigstream parameter. Otherwise
- * @sigstream is assumed to be the signature stream and is used to
- * verify the integirity of the @istream.
+ * Verifies the signature. If @istream is a clearsigned stream, you
+ * should pass %NULL as the @sigstream parameter and may wish to
+ * provide an @ostream argument for GMime to output the original
+ * plaintext into. Otherwise @sigstream is assumed to be the signature
+ * stream and is used to verify the integirity of the @istream.
  *
- * Returns: (transfer full): a #GMimeSignatureList object containing
+ * Returns: (nullable) (transfer full): a #GMimeSignatureList object containing
  * the status of each signature or %NULL on error.
  **/
 GMimeSignatureList *
-g_mime_crypto_context_verify (GMimeCryptoContext *ctx, GMimeDigestAlgo digest, GMimeStream *istream,
-			      GMimeStream *sigstream, GError **err)
+g_mime_crypto_context_verify (GMimeCryptoContext *ctx, GMimeVerifyFlags flags, GMimeStream *istream,
+			      GMimeStream *sigstream, GMimeStream *ostream, GError **err)
 {
 	g_return_val_if_fail (GMIME_IS_CRYPTO_CONTEXT (ctx), NULL);
 	g_return_val_if_fail (GMIME_IS_STREAM (istream), NULL);
 	
-	return GMIME_CRYPTO_CONTEXT_GET_CLASS (ctx)->verify (ctx, digest, istream, sigstream, err);
+	return GMIME_CRYPTO_CONTEXT_GET_CLASS (ctx)->verify (ctx, flags, istream, sigstream, ostream, err);
 }
 
 
 static int
-crypto_encrypt (GMimeCryptoContext *ctx, gboolean sign, const char *userid, GMimeDigestAlgo digest,
+crypto_encrypt (GMimeCryptoContext *ctx, gboolean sign, const char *userid, GMimeEncryptFlags flags,
 		GPtrArray *recipients, GMimeStream *istream, GMimeStream *ostream, GError **err)
 {
-	g_set_error (err, GMIME_ERROR, GMIME_ERROR_NOT_SUPPORTED,
-		     "Encryption is not supported by this crypto context");
+	g_set_error_literal (err, GMIME_ERROR, GMIME_ERROR_NOT_SUPPORTED,
+			     "Encryption is not supported by this crypto context");
 	
 	return -1;
 }
@@ -464,10 +439,9 @@ crypto_encrypt (GMimeCryptoContext *ctx, gboolean sign, const char *userid, GMim
  * g_mime_crypto_context_encrypt:
  * @ctx: a #GMimeCryptoContext
  * @sign: sign as well as encrypt
- * @userid: key id (or email address) to use when signing (assuming @sign is %TRUE)
- * @digest: digest algorithm to use when signing
- * @recipients: (element-type utf8): an array of recipient key ids
- *   and/or email addresses
+ * @userid: (nullable): the key id (or email address) to use when signing (assuming @sign is %TRUE)
+ * @flags: a set of #GMimeEncryptFlags
+ * @recipients: (element-type utf8): an array of recipient key ids and/or email addresses
  * @istream: cleartext input stream
  * @ostream: ciphertext output stream
  * @err: a #GError
@@ -478,34 +452,23 @@ crypto_encrypt (GMimeCryptoContext *ctx, gboolean sign, const char *userid, GMim
  * Returns: %0 on success or %-1 on fail.
  **/
 int
-g_mime_crypto_context_encrypt (GMimeCryptoContext *ctx, gboolean sign, const char *userid, GMimeDigestAlgo digest,
+g_mime_crypto_context_encrypt (GMimeCryptoContext *ctx, gboolean sign, const char *userid, GMimeEncryptFlags flags,
 			       GPtrArray *recipients, GMimeStream *istream, GMimeStream *ostream, GError **err)
 {
 	g_return_val_if_fail (GMIME_IS_CRYPTO_CONTEXT (ctx), -1);
 	g_return_val_if_fail (GMIME_IS_STREAM (istream), -1);
 	g_return_val_if_fail (GMIME_IS_STREAM (ostream), -1);
 	
-	return GMIME_CRYPTO_CONTEXT_GET_CLASS (ctx)->encrypt (ctx, sign, userid, digest, recipients, istream, ostream, err);
+	return GMIME_CRYPTO_CONTEXT_GET_CLASS (ctx)->encrypt (ctx, sign, userid, flags, recipients, istream, ostream, err);
 }
 
 
 static GMimeDecryptResult *
-crypto_decrypt (GMimeCryptoContext *ctx, GMimeStream *istream,
-		GMimeStream *ostream, GError **err)
+crypto_decrypt (GMimeCryptoContext *ctx, GMimeDecryptFlags flags, const char *session_key,
+		GMimeStream *istream, GMimeStream *ostream, GError **err)
 {
-	g_set_error (err, GMIME_ERROR, GMIME_ERROR_NOT_SUPPORTED,
-		     "Decryption is not supported by this crypto context");
-	
-	return NULL;
-}
-
-static GMimeDecryptResult *
-crypto_decrypt_session (GMimeCryptoContext *ctx, const char *session_key,
-			GMimeStream *istream, GMimeStream *ostream,
-			GError **err)
-{
-	g_set_error (err, GMIME_ERROR, GMIME_ERROR_NOT_SUPPORTED,
-		     "Decryption with a session key is not supported by this crypto context");
+	g_set_error_literal (err, GMIME_ERROR, GMIME_ERROR_NOT_SUPPORTED,
+			     "Decrysption is not supported by this crypto context");
 	
 	return NULL;
 }
@@ -514,57 +477,19 @@ crypto_decrypt_session (GMimeCryptoContext *ctx, const char *session_key,
 /**
  * g_mime_crypto_context_decrypt:
  * @ctx: a #GMimeCryptoContext
+ * @flags: a set of #GMimeDecryptFlags
+ * @session_key: (nullable): the session key to use or %NULL
  * @istream: input/ciphertext stream
  * @ostream: output/cleartext stream
  * @err: a #GError
  *
- * Decrypts the ciphertext input stream and writes the resulting
- * cleartext to the output stream.
- *
- * If the encrypted input stream was also signed, the returned
- * #GMimeDecryptResult will have a non-%NULL list of signatures, each with a
- * #GMimeSignatureStatus (among other details about each signature).
- *
- * On success, the returned #GMimeDecryptResult will contain a list of
- * certificates, one for each recipient, that the original encrypted stream
- * was encrypted to.
- *
- * Note: It *may* be possible to maliciously design an encrypted stream such
- * that recursively decrypting it will result in an endless loop, causing
- * a denial of service attack on your application.
- *
- * Returns: (transfer full): a #GMimeDecryptResult on success or %NULL
- * on error.
- **/
-GMimeDecryptResult *
-g_mime_crypto_context_decrypt (GMimeCryptoContext *ctx, GMimeStream *istream,
-			       GMimeStream *ostream, GError **err)
-{
-	g_return_val_if_fail (GMIME_IS_CRYPTO_CONTEXT (ctx), NULL);
-	g_return_val_if_fail (GMIME_IS_STREAM (istream), NULL);
-	g_return_val_if_fail (GMIME_IS_STREAM (ostream), NULL);
-	
-	return GMIME_CRYPTO_CONTEXT_GET_CLASS (ctx)->decrypt (ctx, istream, ostream, err);
-}
-
-/**
- * g_mime_crypto_context_decrypt_session:
- * @ctx: a #GMimeCryptoContext
- * @session_key: session key to use
- * @istream: input/ciphertext stream
- * @ostream: output/cleartext stream
- * @err: a #GError
- *
- * Decrypts the ciphertext input stream using a specific session key
- * and writes the resulting cleartext to the output stream. If
- * @session_key is non-%NULL, but is not valid for the ciphertext, the
- * decryption will fail even if other available secret key material
- * may have been able to decrypt it. If @session_key is %NULL, this
- * does the same thing as g_mime_crypto_context_decrypt().
+ * Decrypts the ciphertext input stream and writes the resulting cleartext
+ * to the output stream.
  *
  * When non-%NULL, @session_key should be a %NULL-terminated string,
  * such as the one returned by g_mime_decrypt_result_get_session_key()
- * from a previous decryption.
+ * from a previous decryption. If the @sesion_key is not valid, decryption
+ * will fail.
  *
  * If the encrypted input stream was also signed, the returned
  * #GMimeDecryptResult will have a non-%NULL list of signatures, each with a
@@ -582,26 +507,22 @@ g_mime_crypto_context_decrypt (GMimeCryptoContext *ctx, GMimeStream *istream,
  * on error.
  **/
 GMimeDecryptResult *
-g_mime_crypto_context_decrypt_session (GMimeCryptoContext *ctx, const char *session_key,
-				       GMimeStream *istream, GMimeStream *ostream,
-				       GError **err)
+g_mime_crypto_context_decrypt (GMimeCryptoContext *ctx, GMimeDecryptFlags flags, const char *session_key,
+			       GMimeStream *istream, GMimeStream *ostream, GError **err)
 {
 	g_return_val_if_fail (GMIME_IS_CRYPTO_CONTEXT (ctx), NULL);
 	g_return_val_if_fail (GMIME_IS_STREAM (istream), NULL);
 	g_return_val_if_fail (GMIME_IS_STREAM (ostream), NULL);
 	
-	if (session_key == NULL)
-		return GMIME_CRYPTO_CONTEXT_GET_CLASS (ctx)->decrypt (ctx, istream, ostream, err);
-	else
-		return GMIME_CRYPTO_CONTEXT_GET_CLASS (ctx)->decrypt_session (ctx, session_key, istream, ostream, err);
+	return GMIME_CRYPTO_CONTEXT_GET_CLASS (ctx)->decrypt (ctx, flags, session_key, istream, ostream, err);
 }
 
 
 static int
 crypto_import_keys (GMimeCryptoContext *ctx, GMimeStream *istream, GError **err)
 {
-	g_set_error (err, GMIME_ERROR, GMIME_ERROR_NOT_SUPPORTED,
-		     "You may not import keys with this crypto");
+	g_set_error_literal (err, GMIME_ERROR, GMIME_ERROR_NOT_SUPPORTED,
+			     "You may not import keys with this crypto");
 	
 	return -1;
 }
@@ -616,7 +537,7 @@ crypto_import_keys (GMimeCryptoContext *ctx, GMimeStream *istream, GError **err)
  * Imports a stream of keys/certificates contained within @istream
  * into the key/certificate database controlled by @ctx.
  *
- * Returns: %0 on success or %-1 on fail.
+ * Returns: the total number of keys imported on success or %-1 on fail.
  **/
 int
 g_mime_crypto_context_import_keys (GMimeCryptoContext *ctx, GMimeStream *istream, GError **err)
@@ -629,11 +550,11 @@ g_mime_crypto_context_import_keys (GMimeCryptoContext *ctx, GMimeStream *istream
 
 
 static int
-crypto_export_keys (GMimeCryptoContext *ctx, GPtrArray *keys,
+crypto_export_keys (GMimeCryptoContext *ctx, const char *keys[],
 		    GMimeStream *ostream, GError **err)
 {
-	g_set_error (err, GMIME_ERROR, GMIME_ERROR_NOT_SUPPORTED,
-		     "You may not export keys with this crypto");
+	g_set_error_literal (err, GMIME_ERROR, GMIME_ERROR_NOT_SUPPORTED,
+			     "You may not export keys with this crypto");
 	
 	return -1;
 }
@@ -642,27 +563,30 @@ crypto_export_keys (GMimeCryptoContext *ctx, GPtrArray *keys,
 /**
  * g_mime_crypto_context_export_keys:
  * @ctx: a #GMimeCryptoContext
- * @keys: (element-type utf8): an array of key ids
+ * @keys: an array of key ids, terminated by a %NULL element
  * @ostream: output stream
  * @err: a #GError
  *
  * Exports the keys/certificates in @keys to the stream @ostream from
  * the key/certificate database controlled by @ctx.
  *
+ * If @keys is %NULL or contains only a %NULL element, then all keys
+ * will be exported.
+ *
  * Returns: %0 on success or %-1 on fail.
  **/
 int
-g_mime_crypto_context_export_keys (GMimeCryptoContext *ctx, GPtrArray *keys,
+g_mime_crypto_context_export_keys (GMimeCryptoContext *ctx, const char *keys[],
 				   GMimeStream *ostream, GError **err)
 {
 	g_return_val_if_fail (GMIME_IS_CRYPTO_CONTEXT (ctx), -1);
 	g_return_val_if_fail (GMIME_IS_STREAM (ostream), -1);
-	g_return_val_if_fail (keys != NULL, -1);
 	
 	return GMIME_CRYPTO_CONTEXT_GET_CLASS (ctx)->export_keys (ctx, keys, ostream, err);
 }
 
 
+
 static void g_mime_decrypt_result_class_init (GMimeDecryptResultClass *klass);
 static void g_mime_decrypt_result_init (GMimeDecryptResult *cert, GMimeDecryptResultClass *klass);
 static void g_mime_decrypt_result_finalize (GObject *object);
@@ -744,7 +668,7 @@ g_mime_decrypt_result_finalize (GObject *object)
 GMimeDecryptResult *
 g_mime_decrypt_result_new (void)
 {
-	return g_object_newv (GMIME_TYPE_DECRYPT_RESULT, 0, NULL);
+	return g_object_new (GMIME_TYPE_DECRYPT_RESULT, NULL);
 }
 
 
@@ -823,7 +747,7 @@ g_mime_decrypt_result_set_signatures (GMimeDecryptResult *result, GMimeSignature
  *
  * Gets a list of signatures if the encrypted stream had also been signed.
  *
- * Returns: (transfer none): a #GMimeSignatureList or %NULL if the
+ * Returns: (nullable) (transfer none): a #GMimeSignatureList or %NULL if the
  * stream was not signed.
  **/
 GMimeSignatureList *
@@ -904,9 +828,9 @@ g_mime_decrypt_result_get_mdc (GMimeDecryptResult *result)
 /**
  * g_mime_decrypt_result_set_session_key:
  * @result: a #GMimeDecryptResult
- * @session_key: a pointer to a null-terminated string representing the session key
+ * @session_key: (nullable): a string representing the session key or %NULL to unset the key
  *
- * Set the session_key to be returned by this decryption result.
+ * Set the session key to be returned by this decryption result.
  **/
 void
 g_mime_decrypt_result_set_session_key (GMimeDecryptResult *result, const char *session_key)
@@ -918,7 +842,7 @@ g_mime_decrypt_result_set_session_key (GMimeDecryptResult *result, const char *s
 		g_free (result->session_key);
 	}
 	
-	result->session_key = g_strdup (session_key);
+	result->session_key = session_key ? g_strdup (session_key) : NULL;
 }
 
 
@@ -926,12 +850,9 @@ g_mime_decrypt_result_set_session_key (GMimeDecryptResult *result, const char *s
  * g_mime_decrypt_result_get_session_key:
  * @result: a #GMimeDecryptResult
  *
- * Get the session_key used for this decryption, if the underlying
- * crypto context is capable of and (configured to) retrieve session
- * keys during decryption.  See, for example,
- * g_mime_crypto_context_set_retrieve_session_key().
+ * Get the session key used for this decryption.
  *
- * Returns: the session_key digest algorithm used, or NULL if no
+ * Returns: (nullable): the session key digest algorithm used, or %NULL if no
  * session key was requested or found.
  **/
 const char *
