@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*  GMime
- *  Copyright (C) 2000-2014 Jeffrey Stedfast
+ *  Copyright (C) 2000-2017 Jeffrey Stedfast
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Lesser General Public License
@@ -185,7 +185,7 @@ g_mime_filter_windows_new (const char *claimed_charset)
 	
 	g_return_val_if_fail (claimed_charset != NULL, NULL);
 	
-	new = g_object_newv (GMIME_TYPE_FILTER_WINDOWS, 0, NULL);
+	new = g_object_new (GMIME_TYPE_FILTER_WINDOWS, NULL);
 	new->claimed_charset = g_strdup (claimed_charset);
 	
 	return (GMimeFilter *) new;
