@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*  GMime
- *  Copyright (C) 2000-2013 Jeffrey Stedfast
+ *  Copyright (C) 2000-2017 Jeffrey Stedfast
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Lesser General Public License
@@ -36,32 +36,9 @@ G_BEGIN_DECLS
 typedef struct _GMimePkcs7Context GMimePkcs7Context;
 typedef struct _GMimePkcs7ContextClass GMimePkcs7ContextClass;
 
-
-/**
- * GMimePkcs7Context:
- * @parent_object: parent #GMimeCryptoContext
- * @priv: private context data
- *
- * A PKCS7 crypto context.
- **/
-struct _GMimePkcs7Context {
-	GMimeCryptoContext parent_object;
-	
-	struct _GMimePkcs7ContextPrivate *priv;
-};
-
-struct _GMimePkcs7ContextClass {
-	GMimeCryptoContextClass parent_class;
-	
-};
-
-
 GType g_mime_pkcs7_context_get_type (void);
 
-GMimeCryptoContext *g_mime_pkcs7_context_new (GMimePasswordRequestFunc request_passwd);
-
-gboolean g_mime_pkcs7_context_get_always_trust (GMimePkcs7Context *ctx);
-void g_mime_pkcs7_context_set_always_trust (GMimePkcs7Context *ctx, gboolean always_trust);
+GMimeCryptoContext *g_mime_pkcs7_context_new (void);
 
 G_END_DECLS
 
