@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*  GMime
- *  Copyright (C) 2000-2014 Jeffrey Stedfast
+ *  Copyright (C) 2000-2017 Jeffrey Stedfast
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Lesser General Public License
@@ -24,6 +24,7 @@
 #endif
 
 #include <glib.h>
+#include <glib/gstdio.h>
 
 #include <sys/types.h>
 #include <sys/stat.h>
@@ -32,6 +33,7 @@
 #include <errno.h>
 
 #include "gmime-stream-fs.h"
+#include "gmime-error.h"
 
 #ifndef HAVE_FSYNC
 #ifdef G_OS_WIN32
@@ -160,7 +162,8 @@ stream_read (GMimeStream *stream, char *buf, size_t len)
 		len = (size_t) MIN (stream->bound_end - stream->position, (gint64) len);
 	
 	/* make sure we are at the right position */
-	lseek (fs->fd, (off_t) stream->position, SEEK_SET);
+	if (lseek (fs->fd, (off_t) stream->position, SEEK_SET) == -1)
+		return -1;
 	
 	do {
 		nread = read (fs->fd, buf, len);
@@ -196,7 +199,8 @@ stream_write (GMimeStream *stream, const char *buf, size_t len)
 		len = (size_t) MIN (stream->bound_end - stream->position, (gint64) len);
 	
 	/* make sure we are at the right position */
-	lseek (fs->fd, (off_t) stream->position, SEEK_SET);
+	if (lseek (fs->fd, (off_t) stream->position, SEEK_SET) == -1)
+		return -1;
 	
 	do {
 		do {
@@ -376,8 +380,11 @@ stream_length (GMimeStream *stream)
 	if (stream->bound_end != -1)
 		return stream->bound_end - stream->bound_start;
 	
-	bound_end = lseek (fs->fd, (off_t) 0, SEEK_END);
-	lseek (fs->fd, (off_t) stream->position, SEEK_SET);
+	if ((bound_end = lseek (fs->fd, (off_t) 0, SEEK_END)) == -1)
+		return -1;
+	
+	if (lseek (fs->fd, (off_t) stream->position, SEEK_SET) == -1)
+		return -1;
 	
 	if (bound_end < stream->bound_start) {
 		errno = EINVAL;
@@ -392,9 +399,9 @@ stream_substream (GMimeStream *stream, gint64 start, gint64 end)
 {
 	GMimeStreamFs *fs;
 	
-	fs = g_object_newv (GMIME_TYPE_STREAM_FS, 0, NULL);
-	g_mime_stream_construct (GMIME_STREAM (fs), start, end);
-	fs->fd = GMIME_STREAM_FS (stream)->fd;
+	fs = g_object_new (GMIME_TYPE_STREAM_FS, NULL);
+	g_mime_stream_construct ((GMimeStream *) fs, start, end);
+	fs->fd = ((GMimeStreamFs *) stream)->fd;
 	fs->owner = FALSE;
 	fs->eos = FALSE;
 	
@@ -423,8 +430,8 @@ g_mime_stream_fs_new (int fd)
 	if ((start = lseek (fd, (off_t) 0, SEEK_CUR)) == -1)
 		start = 0;
 	
-	fs = g_object_newv (GMIME_TYPE_STREAM_FS, 0, NULL);
-	g_mime_stream_construct (GMIME_STREAM (fs), start, -1);
+	fs = g_object_new (GMIME_TYPE_STREAM_FS, NULL);
+	g_mime_stream_construct ((GMimeStream *) fs, start, -1);
 	fs->owner = TRUE;
 	fs->eos = FALSE;
 	fs->fd = fd;
@@ -453,8 +460,8 @@ g_mime_stream_fs_new_with_bounds (int fd, gint64 start, gint64 end)
 	_setmode (fd, O_BINARY);
 #endif
 	
-	fs = g_object_newv (GMIME_TYPE_STREAM_FS, 0, NULL);
-	g_mime_stream_construct (GMIME_STREAM (fs), start, end);
+	fs = g_object_new (GMIME_TYPE_STREAM_FS, NULL);
+	g_mime_stream_construct ((GMimeStream *) fs, start, end);
 	fs->owner = TRUE;
 	fs->eos = FALSE;
 	fs->fd = fd;
@@ -464,27 +471,28 @@ g_mime_stream_fs_new_with_bounds (int fd, gint64 start, gint64 end)
 
 
 /**
- * g_mime_stream_fs_new_for_path:
+ * g_mime_stream_fs_open:
  * @path: the path to a file
  * @flags: as in open(2)
  * @mode: as in open(2)
+ * @err: a #GError
  *
  * Creates a new #GMimeStreamFs object for the specified @path.
  *
- * Returns: a stream using for reading and/or writing to the specified
- * file path or %NULL on error.
- *
- * Since: 2.6.18
+ * Returns: (transfer full): a stream using for reading and/or writing to the
+ * specified file path or %NULL on error.
  **/
 GMimeStream *
-g_mime_stream_fs_new_for_path (const char *path, int flags, int mode)
+g_mime_stream_fs_open (const char *path, int flags, int mode, GError **err)
 {
 	int fd;
 	
 	g_return_val_if_fail (path != NULL, NULL);
 	
-	if ((fd = g_open (path, flags, mode)) == -1)
+	if ((fd = g_open (path, flags, mode)) == -1) {
+		g_set_error (err, GMIME_ERROR, errno, "Failed to open `%s': %s", path, g_strerror (errno));
 		return NULL;
+	}
 	
 	return g_mime_stream_fs_new (fd);
 }
