@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*  GMime
- *  Copyright (C) 2000-2014 Jeffrey Stedfast
+ *  Copyright (C) 2000-2017 Jeffrey Stedfast
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -23,6 +23,8 @@
 #endif
 
 #include <glib.h>
+#include <glib/gstdio.h>
+
 #include <stdlib.h>
 #ifdef ENABLE_THREADS
 #include <pthread.h>
@@ -360,24 +362,78 @@ g_throw (Exception *ex)
 
 #define v2_1_16 ((2 << 24) | (1 << 16) | (16 << 8))
 
-int _g_mime_get_gpg_version (const char *gpg);
+static int
+get_gpg_version (const char *path)
+{
+	const char vheader[] = " (GnuPG) ";
+	int v, n = 0, version = 0;
+	const char *inptr;
+	char buffer[128];
+	char *command;
+	FILE *gpg;
+	
+	g_return_val_if_fail (path != NULL, -1);
+	
+	command = g_strdup_printf ("%s --version", path);
+	gpg = popen (command, "r");
+	g_free (command);
+	
+	if (gpg == NULL)
+		return -1;
+	
+	inptr = fgets (buffer, 128, gpg);
+	pclose (gpg);
 
-int
+	if (!(inptr = strstr (inptr, vheader)))
+		return -1;
+	
+	inptr += sizeof (vheader) - 1;
+	while (*inptr >= '0' && *inptr <= '9' && n < 4) {
+		v = 0;
+		
+		while (*inptr >= '0' && *inptr <= '9' && (v < 25 || (v == 25 && *inptr < '6'))) {
+			v = (v * 10) + (*inptr - '0');
+			inptr++;
+		}
+		
+		version = (version << 8) + v;
+		n++;
+		
+		if (*inptr != '.')
+			break;
+		
+		inptr++;
+	}
+	
+	if (n == 0)
+		return -1;
+	
+	if (n < 4)
+		version = version << ((4 - n) * 8);
+	
+	return version;
+}
+
+gboolean
 testsuite_can_safely_override_session_key (const char *gpg)
 {
-	return _g_mime_get_gpg_version (gpg) >= v2_1_16;
+	return get_gpg_version (gpg) >= v2_1_16;
 }
 
 int
 testsuite_setup_gpghome (const char *gpg)
 {
-	const char directive[] = "pinentry-mode loopback\n";
-	char *command;
+	const char *files[] = { "./tmp/.gnupg/gpg.conf", "./tmp/.gnupg/gpgsm.conf", "./tmp/.gnupg/gpg-agent.conf", "./tmp/.gnupg/dirmngr.conf", NULL };
+	const char debug[] = "log-file socket://%s/tmp/.gnupg/S.log\ndebug 1024\nverbose\n";
+	char *command, *cwd;
+	FILE *fp;
+	int i;
 	
 	/* reset .gnupg config directory */
 	if (system ("/bin/rm -rf ./tmp") != 0)
 		return EXIT_FAILURE;
-	if (system ("/bin/mkdir ./tmp") != 0)
+	
+	if (g_mkdir ("./tmp", 0755) != 0)
 		return EXIT_FAILURE;
 	
 	g_setenv ("GNUPGHOME", "./tmp/.gnupg", 1);
@@ -395,13 +451,11 @@ testsuite_setup_gpghome (const char *gpg)
 	
 	g_free (command);
 	
-	if (_g_mime_get_gpg_version (gpg) >= ((2 << 24) | (1 << 16))) {
-		FILE *fp;
-		
-		if ((fp = fopen ("./tmp/.gnupg/gpg.conf", "w")) == NULL)
+	if (get_gpg_version (gpg) >= ((2 << 24) | (1 << 16))) {
+		if (!(fp = fopen ("./tmp/.gnupg/gpg.conf", "a")))
 			return EXIT_FAILURE;
 		
-		if (fwrite (directive, sizeof (directive) - 1, 1, fp) != 1) {
+		if (fprintf (fp, "pinentry-mode loopback\n") == -1) {
 			fclose (fp);
 			return EXIT_FAILURE;
 		}
@@ -410,6 +464,39 @@ testsuite_setup_gpghome (const char *gpg)
 			return EXIT_FAILURE;
 	}
 	
+	if (!(fp = fopen ("./tmp/.gnupg/gpgsm.conf", "a")))
+		return EXIT_FAILURE;
+	
+	if (fprintf (fp, "disable-crl-checks\n") == -1) {
+		fclose (fp);
+		return EXIT_FAILURE;
+	}
+	
+	if (fclose (fp))
+		return EXIT_FAILURE;
+	
+#if DEBUG_GNUPG
+	cwd = g_get_current_dir ();
+	
+	for (i = 0; files[i]; i++) {
+		if (!(fp = fopen (files[i], "a")))
+			return EXIT_FAILURE;
+		
+		if (fprintf (fp, debug, cwd) == -1) {
+			g_free (cwd);
+			fclose (fp);
+			return EXIT_FAILURE;
+		}
+		
+		if (fclose (fp)) {
+			g_free (cwd);
+			return EXIT_FAILURE;
+		}
+	}
+	
+	g_free (cwd);
+#endif
+	
 	return EXIT_SUCCESS;
 }
 
